#include <winver.h>
#include "version.h"

VS_VERSION_INFO VERSIONINFO
FILEVERSION     MAJOR_VERSION, MINOR_VERSION, PATCH_VERSION, 0
PRODUCTVERSION  MAJOR_VERSION, MINOR_VERSION, PATCH_VERSION, 0
FILEOS          VOS_NT_WINDOWS32
FILETYPE        VFT_APP
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "000004B0"
        BEGIN
            VALUE "FileVersion",      APP_VERSION "." BUILD_VERSION
            VALUE "ProductVersion",   APP_VERSION "." BUILD_VERSION
            VALUE "FileDescription",  "DB Browser for SQLite"
            VALUE "ProductName",      "DB Browser for SQLite"
            VALUE "InternalName",     "DB Browser for SQLite"
            VALUE "OriginalFilename", "DB Browser for SQLite.exe"
            VALUE "CompanyName",      "DB Browser for SQLite Team"
            VALUE "LegalCopyright",   "Copyright © DB Browser for SQLite Team"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x0000, 0x04B0
    END
END

IDI_DB4S ICON "iconwin.ico"
